import {
  Directive,
  ElementRef,
  EventEmitter,
  HostListener,
  Inject,
  Input,
  LOCALE_ID,
  OnChanges,
  Output,
  Renderer2,
  SimpleChanges
} from '@angular/core';
import {NgModel} from '@angular/forms';
import {VersionedSection} from 'facade-api';
import {ObjectHelper} from './model-wrappers/objectHelper';
import {TooltipDirective} from './tooltip/tooltip.directive';
import {WindowService} from './services/window.service';
import {DocumentService} from './services/document.service';
import {formatDate} from '@angular/common';

@Directive({
  selector: '[ngModel][appVersionedSection]',
  providers: [NgModel]
})
export class VersionedSectionDirective extends TooltipDirective implements OnChanges{
  @Input() appVersionedSection?: VersionedSection;
  @Output() appVersionedSectionChange: EventEmitter<VersionedSection> = new EventEmitter<VersionedSection>();

  constructor(el: ElementRef,
              renderer: Renderer2,
              windowService: WindowService,
              documentService: DocumentService,
              @Inject(LOCALE_ID) private locale: string) {
    super(el, renderer, windowService, documentService);
    this.delay = 200;
    this.placement = 'cursor';
  }

  public ngOnChanges(changes: SimpleChanges): void {
    if (changes.appVersionedSection) {
      this.updateTooltip();
    }
  }

  @HostListener('ngModelChange')
  public onChange(): void {
    if (this.el.nativeElement?.classList.contains('ng-dirty')) {
      const result = ObjectHelper.setOnPath(this.appVersionedSection, 'dirty', true);
      this.updateTooltip();
      this.appVersionedSectionChange.emit(result);
    }
  }

  private updateTooltip(): void {
    this.tooltipClasses = undefined;
    if (this.appVersionedSection?.dirty) {
      this.appTooltip = 'Versionsgruppen er ændret af dig og endnu ikke gemt';
    } else if (this.appVersionedSection?.author?.name || this.appVersionedSection?.modifiedTime) {
      this.appTooltip = this.appVersionedSection?.author?.name;
      if (this.appVersionedSection?.modifiedTime) {
        this.appTooltip += '\n' + formatDate(this.appVersionedSection.modifiedTime, `dd.MM.yyyy '- kl.' hh.mm`, this.locale);
      }
    } else {
      this.tooltipClasses = 'hidden';
    }
  }
}
